#!/usr/bin/env python

# split_on_guides.py - a Python-fu script which splits an image along its guides
# Version 1.0
#
# Copyright (C) 2003  Dwayne C. Litzenberger <dlitz@dlitz.net>
#
# * This work shall enter the public domain on the first day of the tenth
# * (10th) year following the copyright year listed above.  Until then,
# * the following terms shall apply:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from gimpfu import *
import os
import sys
import cgi, urlparse

if not hasattr(__builtins__, 'enumerate'):
    def enumerate(iterable):
        i = 0
        retval = []
        for v in iterable:
            retval.append((i, v))
            i += 1
        return retval

def guides_to_rectangles(timg):
    guides = [{}, {}]  # We are emulating Python 2.3's Set behaviour
    guide = timg.find_next_guide(0)
    if not guide:
        raise ValueError, "image must contain at least one guide"
    while guide:
        orientation = timg.get_guide_orientation(guide)
        position = timg.get_guide_position(guide)
        if position != 0:   # 0 is implied, so we don't want to add it
            guides[orientation][position] = None
        guide = timg.find_next_guide(guide)
    guides[0][timg.height] = None
    guides[1][timg.width] = None
    guides[0] = guides[0].keys()    # more Set behaviour emulation
    guides[1] = guides[1].keys()
    guides[0].sort()    # now treating guides[...] as lists, rather than Sets
    guides[1].sort()
    
    rows = []
    y = 0
    for h in guides[0]:
        x = 0
        cols = []
        for v in guides[1]:
            cols.append((x, y, v - 1, h - 1))
            x = v
        rows.append(cols)
        y = h
    
    return rows

def split_on_guides(timg, tdrawable, leave_open=True, save_files=False,
                    image_directory=None, filename_template=None,
                    file_extensions=None, save_html=False, html_filename=None,
                    image_baseurl="", save_info=False, info_filename=None):
    if not (leave_open or save_files or save_html or save_info):
        raise TypeError, "cowardly refusing to do nothing"
    if (info_filename or save_html or save_files) and not file_extensions:
        raise TypeError, "info_filename, save_html and save_files require at least one file extension"
    if image_baseurl and not image_baseurl.endswith("/"):
        image_baseurl += "/"
    pdb.gimp_image_undo_freeze(timg)
    saved_selection = None
    try:
        #assert(tdrawable == pdb.gimp_image_active_drawable(timg))
        
        # Split file_extensions string
        if file_extensions:
            if "," in file_extensions:
                file_extensions = file_extensions.split(",")
            elif " " in file_extensions:
                file_extensions = file_extensions.split(" ")
            else:
                file_extensions = [file_extensions]
            file_extensions = filter(None,
                [x.strip() for x in file_extensions])
        
        # Get image attributes
        image_base_type = pdb.gimp_image_base_type(timg)
        drawable_type = pdb.gimp_drawable_type(tdrawable)
        has_alpha = pdb.gimp_drawable_has_alpha(tdrawable)
        cmap = pdb.gimp_image_get_cmap(timg)
        
        # Save the current selection
        saved_selection = pdb.gimp_selection_save(timg)
        
        rows = guides_to_rectangles(timg)
        numrows = len(rows)
        numcols = len(rows[0])
        total_images = numrows * numcols
    
        if save_html:
            html_file = open(html_filename, "w")
            html_file.write('''<table cellpadding="0" cellspacing="0">''')
        
        if save_info:
            info_file = open(info_filename, "w")
            
        i = 0
        try: gimp.progress_init("Splitting...")
        except: pass
        for rownum, row in enumerate(rows):
            if save_html: html_file.write('''<!--\nNew Row --><tr>''')
            for colnum, (x,y,x2,y2) in enumerate(row):
                w = x2 - x
                h = y2 - y
                pdb.gimp_rect_select(timg,x, y, w, h, REPLACE, 0, 0)
                pdb.gimp_edit_copy(tdrawable)
                newimg = pdb.gimp_image_new(w, h, image_base_type)
                newdrawable = pdb.gimp_layer_new(newimg, w, h, drawable_type,
                    "Background", 100, NORMAL_MODE)
                if cmap:
                    pdb.gimp_image_set_cmap(newimg, *cmap)
                if has_alpha:
                    pdb.gimp_drawable_fill(newdrawable, TRANS_IMAGE_FILL)
                pdb.gimp_image_add_layer(newimg, newdrawable, 1)
                
                floating_sel = pdb.gimp_edit_paste(newdrawable, 0)
                pdb.gimp_floating_sel_anchor(floating_sel)
                
                if save_files and filename_template and file_extensions:
                    for ext in file_extensions:
                        filename = filename_template % {
                                    'row': rownum,
                                    'col': colnum,
                                    'ext': ext,
                                }

                        pdb.gimp_file_save(
                            newimg,
                            newdrawable,
                            os.path.join(image_directory, filename),
                            filename)
                if save_info:
                    for ext in file_extensions:
                        filename = filename_template % {
                                    'row': rownum,
                                    'col': colnum,
                                    'ext': ext,
                                }
                        info_file.write(
                            "%d %d %d %d %s\n" % (rownum, colnum, w, h, filename))
                if save_html:
                    filename = filename_template % {
                                'row': rownum,
                                'col': colnum,
                                'ext': file_extensions[0],
                            }
                    html_file.write(
                        '''<td
                            width="%(width)s" height="%(height)s"
                            style="width: %(width)spx; height: %(height)spx"
                        ><img
                            src="%(src)s"
                            alt=""
                            width="%(width)s" height="%(height)s"
                            style="width: %(width)spx; height: %(height)spx"
                        /></td>''' % {
                            'width': str(w),
                            'height': str(h),
                            'src':
                                cgi.escape(
                                    urlparse.urljoin(image_baseurl, filename)),
                        })
                    
                if leave_open:
                    pdb.gimp_display_new(newimg)
                else:
                    #pdb.gimp_layer_delete(newdrawable)
                    pdb.gimp_image_delete(newimg)
                i += 1
                try: gimp.progress_update(float(i) / total_images)
                except: pass
            if save_html: html_file.write('''</tr>''')
        if save_html:
            html_file.write("</table>")
            del html_file
    finally:       
        if saved_selection:
            # Restore and delete the saved selection
            pdb.gimp_selection_load(saved_selection)
            pdb.gimp_image_remove_channel(timg, saved_selection)
            pdb.gimp_channel_delete(saved_selection)
        pdb.gimp_image_undo_thaw(timg)

class ExceptionWrapper:
    def __init__(self, callable):
        self.callable = callable
    def __call__(self, *args, **kwargs):
        try:
            return self.callable(*args, **kwargs)
        except:
            pdb.gimp_message(str(sys.exc_info()[0]) + ": " + str(sys.exc_info()[1]))
            raise

register(
    "python_fu_split_on_guides",            # name
    "Split an image along its guides",      # blurb
    "Split an image along its guides",      # help
    "Dwayne C. Litzenberger",               # author
    "Dwayne C. Litzenberger",               # copyright
    "2003",                                 # date
    "<Image>/Guides/Split on Guides",       # menupath
    "*",                                    # imagetypes
    [                                       # params
        (PF_BOOL, "leave_open", "Leave images open?", False),
        (PF_BOOL, "save_files", "Save files?", False),
        (PF_STRING, "image_directory", "Image directory", os.getcwd()),
        (PF_STRING, "filename_template",
            "Image filename template\n" + 
                "%(row)s - row number\n" +
                "%(col)s - column number\n" +
                "%(ext)s - file extension",
            "img_%(row)s_%(col)s.%(ext)s"),
        (PF_STRING, "file_extensions",
            "File types to save (separated by spaces)", 'png'),
        (PF_BOOL, "save_html", "Save an HTML table?", False),
        (PF_STRING, "html_filename", "Filename for HTML file",
            os.path.join(os.getcwd(), "img.html")),
        (PF_STRING, "image_baseurl", "Base image URL for HTML file", ""),
        (PF_BOOL, "save_info",
            "Save an image info file?\n" +
                "Consists of one image per line:\n" +
                "row col width height filename",
            False),
        (PF_STRING, "info_filename", "Filename for info file",
            os.path.join(os.getcwd(), "img.txt")),
    ],
    [],                                     # results
    ExceptionWrapper(split_on_guides))      # function

main()
